/**
 * Copyright (c) 2012 Nokia Corporation.
 */

import QtQuick 1.1
import com.nokia.symbian 1.1
import CustomElements 1.0

Item {
    id: cameraView

    property CustomCamera camera
    property string currentDevice
    property bool frontCamera: true

    property real horizonValue
    property alias imageAnalyzer: imageAnalyzer

    property string lastCapturedImage

    property real controlOpacity

    signal settingsPressed()
    signal requestCameraView()

    /*!
      Indicates the lock status on the reticule.
    */
    function indicateLockStatus(status) {
        if (status === "failed")
            reticule.source = "../images/crosshair_frame_red.svg";
        else if (status === "locked")
            reticule.source = "../images/crosshair_frame_green.svg";
    }

    /*!
      Sets the reticule color back to white.
    */
    function resetReticuleColor() {
        reticule.source = "../images/crosshair_frame_white.svg"
    }

    /*!
      Closes the IsoSelector if it is in expanded mode.
    */
    function resetView() {
        isoSelector.opacity = 0;
        flashSelector.opacity = 0;
        exposureCompensationSelector.opacity = 0;
        reticule.opacity = 1;
    }

    /*!
      Starts the primary camera.
    */
    function start() {
        var devices = camera.availableDevices;
        if (devices.length > 1) {
            cameraToggle.visible = true;
        }

        // Start primary camera
        currentDevice = devices[0];
        camera.start(currentDevice);
    }

    /*!
      Switches the camera to front or primary.
    */
    function toggleCamera() {
        var devices = camera.availableDevices;
        if (devices.length < 2)
            return;

        resetReticuleColor();

        if (cameraView.currentDevice === devices[0]) {
            cameraView.currentDevice = devices[1];
            cameraView.frontCamera = false;
        }
        else {
            cameraView.currentDevice = devices[0];
            cameraView.frontCamera = true;
        }

        camera.start(cameraView.currentDevice);
    }

    /*!
      Performs the auto focus.
    */
    function focusToCoordinate(x, y) {
        // Shutter button can be pressed while in settings or info view
        // and we should switch to the camera view.
        cameraView.requestCameraView();

        resetView();
        resetReticuleColor();

        camera.focusToCoordinate(x, y);
    }

    /*!
      Requests camera to capture a image.
    */
    function captureImage() {
        camera.captureImage();
    }

    focus: true

    Keys.onPressed: {
        if (event.key === Qt.Key_VolumeUp) {
            // Zoom down
            zoomSlider.value -= 1;
            event.accepted = true;
        }
        else if (event.key === Qt.Key_VolumeDown) {
            // Zoom up
            zoomSlider.value += 1;
            event.accepted = true;
        }
        else if (event.key === 17825824 && !event.isAutoRepeat) {
            // Shutter pressed
            cameraView.captureImage();
            event.accepted = true;
        }
        else if (event.key === 17825825 && !event.isAutoRepeat) {
            // Shutter pressed halfway
            cameraView.focusToCoordinate(0, 0);
            event.accepted = true;
        }
    }

    MouseArea {
        id: focusArea

        anchors.fill: parent

        onClicked: { cameraView.focusToCoordinate(mouse.x, mouse.y); }
    }

    ImageAnalyzer {
        id: imageAnalyzer
    }

    HistogramDisplay {
        id: histogramDisplay

        anchors {
            bottom: parent.bottom; bottomMargin: 8
            right: parent.right; rightMargin: 10
        }

        width: 100
        height: 50

        imageAnalyzer: imageAnalyzer
    }

    Image {
        id: reticule

        anchors.centerIn: parent

        source: "../images/crosshair_frame_white.svg"

        Behavior on opacity { PropertyAnimation { duration: 250 } }

        Image {
            id: waterLevelReticule

            anchors.centerIn: parent

            smooth: true
            source: "../images/crosshair_white.svg"
            rotation: cameraView.horizonValue
        }
    }

    /*
    PerformanceMeter {
        id: performanceMeter

        anchors {
            top: parent.top
            right: parent.center
        }

        width: 128; height: 64
    }
    */

    Slider {
        id: zoomSlider

        anchors {
            top: parent.top; topMargin: 10
            bottom: parent.bottom; bottomMargin: 80
            left: parent.left; leftMargin: 10
        }

        opacity: cameraView.controlOpacity

        orientation: Qt.Vertical
        inverted: true
        value: camera.digitalZoom
        minimumValue: 1
        maximumValue: camera.maximumDigitalZoom
        onValueChanged: { camera.zoomTo(1, value); }
    }

    CustomButton  {
        id: shootButton

        anchors {
            verticalCenter: parent.verticalCenter
            right: parent.right; rightMargin: 10
        }

        width: 60; height: 60

        opacity: cameraView.controlOpacity
        iconSource: "../images/shooting.svg"

        onClicked: { cameraView.captureImage(); }
    }

    ToolButton {
        id: settings

        anchors {
            horizontalCenter: parent.horizontalCenter
            top: parent.top; topMargin: 10
        }

        opacity: cameraView.controlOpacity
        visible: cameraView.frontCamera

        iconSource: "toolbar-settings"
        onClicked: {
            cameraView.resetView();
            cameraView.settingsPressed();
        }
    }

    CustomButton {
        id: cameraToggle

        anchors {
            top: parent.top; topMargin: 10
            right: parent.right; rightMargin: 10
        }

        opacity: cameraView.controlOpacity
        iconSource: "../images/camera_switch.svg"
        onClicked: { cameraView.toggleCamera(); }
    }

    Item {
        id: bottomRow

        anchors {
            left: parent.left; leftMargin: 10
            right: parent.right; rightMargin: 10
            bottom: parent.bottom; bottomMargin: 10
        }

        height: backButton.height

        ToolButton {
            id: backButton

            anchors {
                left: parent.left
                bottom: parent.bottom
            }

            opacity: cameraView.controlOpacity
            iconSource: "toolbar-back"
            onClicked: { Qt.quit(); }
        }

        FlashSelector {
            id: flashSelector

            anchors {
                bottom: flashModesButton.top; bottomMargin: 5
                horizontalCenter: flashModesButton.horizontalCenter
            }

            opacity: 0
            controlOpacity: cameraView.controlOpacity
            model: camera.supportedFlashModes
            onFlashSelected: { camera.flashMode = flashMode; }
        }

        CustomButton {
            id: flashModesButton

            anchors {
                left: backButton.right; leftMargin: 42
                bottom: parent.bottom
            }

            opacity: cameraView.controlOpacity
            visible: cameraView.frontCamera
            iconSource: flashSelector.resolveImageFromMode(camera.flashMode)

            onClicked: {
                if (flashSelector.opacity == 0) {
                    cameraView.resetView();
                    reticule.opacity = 0;
                    flashSelector.opacity = 1;
                }
                else {
                    cameraView.resetView();
                }
            }
        }

        ExposureCompensationSelector {
            id: exposureCompensationSelector

            anchors {
                bottom: exposureCompensation.top; bottomMargin: 5
                horizontalCenter: exposureCompensation.horizontalCenter
            }

            width: 200
            height: 200

            opacity: 0
            controlOpacity: cameraView.controlOpacity
            onValueChanged: { camera.exposureCompensation = value; }
        }

        CustomButton {
            id: exposureCompensation

            anchors {
                left: flashModesButton.right; leftMargin: 44
                bottom: parent.bottom
            }

            opacity: cameraView.controlOpacity
            visible: cameraView.frontCamera
            iconSource: "../images/light_exposure.svg"

            onClicked: {
                if (exposureCompensationSelector.opacity == 0) {
                    cameraView.resetView();
                    reticule.opacity = 0;
                    exposureCompensationSelector.opacity = 1;
                }
                else {
                    cameraView.resetView();
                }
            }
        }

        IsoSelector {
            id: isoSelector

            anchors {
                bottom: isoSelectorButton.top; bottomMargin: 5
                horizontalCenter: isoSelectorButton.horizontalCenter
            }

            opacity: 0
            controlOpacity: cameraView.controlOpacity
            model: camera.supportedIsoValues
            onIsoSelected: { camera.isoValue = isoValue; }
        }

        CustomButton {
            id: isoSelectorButton

            anchors {
                horizontalCenter: parent.horizontalCenter
                bottom: parent.bottom
            }

            opacity: cameraView.controlOpacity
            visible: cameraView.frontCamera
            iconSource: "../images/iso_" + camera.isoValue + ".svg"
            onClicked: {
                if (isoSelector.opacity == 0) {
                    cameraView.resetView();
                    reticule.opacity = 0;
                    isoSelector.opacity = 1;
                }
                else {
                    cameraView.resetView();
                }
            }
        }

        CustomButton {
            id: galleryButton

            anchors {
                left: isoSelectorButton.right; leftMargin: 80
                bottom: parent.bottom
            }

            opacity: cameraView.controlOpacity
            visible: cameraView.lastCapturedImage !== ""
            iconSource: "../images/image_gallery.svg"

            onClicked: {
                Qt.openUrlExternally("file:///" + cameraView.lastCapturedImage);
            }
        }
    }
}
